# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import datetime
import struct
from argparse import Namespace
from contextlib import redirect_stdout
from io import StringIO
from pathlib import Path
from unittest import TestCase
from unittest.mock import MagicMock, patch

from pontos.errors import PontosError
from pontos.testing import temp_directory, temp_file
from pontos.updateheader.updateheader import _add_header as add_header
from pontos.updateheader.updateheader import (
    _compile_copyright_regex,
    main,
    parse_args,
    update_file,
)
from pontos.updateheader.updateheader import (
    _compile_outdated_regex as compile_outdated_regex,
)
from pontos.updateheader.updateheader import _find_copyright as find_copyright
from pontos.updateheader.updateheader import (
    _get_exclude_list as get_exclude_list,
)
from pontos.updateheader.updateheader import (
    _get_modified_year as get_modified_year,
)
from pontos.updateheader.updateheader import (
    _remove_outdated_lines as remove_outdated_lines,
)

HEADER = """# SPDX-FileCopyrightText: {date} Greenbone AG
#
# SPDX-License-Identifier: AGPL-3.0-or-later"""


class GetModifiedYearTestCase(TestCase):
    @patch("pontos.updateheader.updateheader.Git")
    def test_get_modified_year(self, git_mock):
        with temp_file(name="test.py", change_into=True) as test_file:
            git_instance_mock = MagicMock()
            git_instance_mock.log.return_value = ["2020"]
            git_mock.return_value = git_instance_mock

            year = get_modified_year(f=test_file)
            self.assertEqual(year, "2020")

    def test_get_modified_year_error(self):
        with (
            temp_directory(change_into=True) as temp_dir,
            self.assertRaises(PontosError),
        ):
            test_file = temp_dir / "test.py"

            get_modified_year(f=test_file)


class FindCopyRightTestCase(TestCase):
    def setUp(self) -> None:
        self.company = "Greenbone AG"
        self.regex = _compile_copyright_regex()

    def test_find_copyright(self):
        test_line = "# Copyright (C) 1995-2021 Greenbone AG"
        test2_line = "# Copyright (C) 1995 Greenbone AG"
        invalid_line = (
            "# This program is free software: "
            "you can redistribute it and/or modify"
        )

        # Full match
        found, match = find_copyright(
            copyright_regex=self.regex, line=test_line
        )
        self.assertTrue(found)
        self.assertIsNotNone(match)
        self.assertEqual(match.creation_year, "1995")
        self.assertEqual(match.modification_year, "2021")
        self.assertEqual(match.company, self.company)

        # No modification Date
        found, match = find_copyright(
            copyright_regex=self.regex, line=test2_line
        )
        self.assertTrue(found)
        self.assertIsNotNone(match)
        self.assertEqual(match.creation_year, "1995")
        self.assertIsNone(match.modification_year)
        self.assertEqual(match.company, self.company)

        # No match
        found, match = find_copyright(
            copyright_regex=self.regex, line=invalid_line
        )
        self.assertFalse(found)
        self.assertIsNone(match)

    def test_find_spdx_copyright(self):
        test_line = "# SPDX-FileCopyrightText: 1995-2021 Greenbone AG"
        test2_line = "# SPDX-FileCopyrightText: 1995 Greenbone AG"
        invalid_line = (
            "# This program is free software: "
            "you can redistribute it and/or modify"
        )

        # Full match
        found, match = find_copyright(
            copyright_regex=self.regex, line=test_line
        )
        self.assertTrue(found)
        self.assertIsNotNone(match)
        self.assertEqual(match.creation_year, "1995")
        self.assertEqual(match.modification_year, "2021")
        self.assertEqual(match.company, self.company)

        # No modification Date
        found, match = find_copyright(
            copyright_regex=self.regex, line=test2_line
        )
        self.assertTrue(found)
        self.assertIsNotNone(match)
        self.assertEqual(match.creation_year, "1995")
        self.assertIsNone(match.modification_year)
        self.assertEqual(match.company, self.company)

        # No match
        found, match = find_copyright(
            copyright_regex=self.regex, line=invalid_line
        )
        self.assertFalse(found)
        self.assertIsNone(match)


class AddHeaderTestCase(TestCase):
    def setUp(self):
        self.company = "Greenbone AG"

    def test_add_header(self):
        expected_header = HEADER.format(date="2021") + "\n"

        header = add_header(
            suffix=".py",
            license_id="AGPL-3.0-or-later",
            company=self.company,
            year="2021",
        )

        self.assertEqual(header, expected_header)

    def test_add_header_wrong_file_suffix(self):
        with self.assertRaises(ValueError):
            add_header(
                suffix=".prr",
                license_id="AGPL-3.0-or-later",
                company=self.company,
                year="2021",
            )

    def test_add_header_license_not_found(self):
        with self.assertRaises(FileNotFoundError):
            add_header(
                suffix=".py",
                license_id="AAAGPL-3.0-or-later",
                company=self.company,
                year="2021",
            )


class UpdateFileTestCase(TestCase):
    maxDiff = None

    def setUp(self):
        self.company = "Greenbone AG"

        self.path = Path(__file__).parent

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_file_not_existing(self, mock_stdout):
        year = "2020"
        license_id = "AGPL-3.0-or-later"

        with temp_directory(change_into=True) as temp_dir:
            test_file = temp_dir / "test.py"

            with self.assertRaises(FileNotFoundError):
                update_file(
                    test_file,
                    year,
                    license_id,
                    self.company,
                )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                ret,
                f"{test_file}: File is not existing.\n",
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_file_wrong_license(self, mock_stdout):
        year = "2020"
        license_id = "AAAGPL-3.0-or-later"

        with temp_file(name="test.py", change_into=True) as test_file:
            update_file(
                test_file,
                year,
                license_id,
                self.company,
            )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                ret,
                f"{test_file}: License file for "
                "AAAGPL-3.0-or-later is not existing.\n",
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_file_suffix_invalid(self, mock_stdout):
        year = "2020"
        license_id = "AGPL-3.0-or-later"

        with temp_file(name="test.pppy", change_into=True) as test_file:
            update_file(
                test_file,
                year,
                license_id,
                self.company,
            )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                ret,
                f"{test_file}: No license header for the format .pppy found.\n",
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_file_binary_file(self, mock_stdout):
        year = "2020"
        license_id = "AGPL-3.0-or-later"

        # create a Binary file ...
        # https://stackoverflow.com/a/30148554
        data = struct.pack(">if", 42, 2.71828182846)

        with (
            temp_file(name="test.py", content=data) as test_file,
            self.assertRaises(UnicodeDecodeError),
        ):
            update_file(
                test_file,
                year,
                license_id,
                self.company,
            )

        ret = mock_stdout.getvalue()
        self.assertEqual(
            ret,
            f"{test_file}: Ignoring binary file.\n",
        )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_create_header(self, mock_stdout):
        year = "1995"
        license_id = "AGPL-3.0-or-later"

        expected_header = HEADER.format(date="1995") + "\n\n"

        with temp_file(name="test.py", change_into=True) as test_file:
            update_file(
                test_file,
                year,
                license_id,
                self.company,
            )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                f"{test_file}: Added license header.\n",
                ret,
            )
            self.assertEqual(
                expected_header, test_file.read_text(encoding="utf-8")
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_create_header_single_year(self, mock_stdout):
        year = "1995"
        license_id = "AGPL-3.0-or-later"

        expected_header = HEADER.format(date="1995") + "\n\n"

        with temp_file(name="test.py", change_into=True) as test_file:
            update_file(
                test_file, year, license_id, self.company, single_year=True
            )
            ret = mock_stdout.getvalue()
            self.assertEqual(
                f"{test_file}: Added license header.\n",
                ret,
            )
            self.assertEqual(
                expected_header, test_file.read_text(encoding="utf-8")
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_header_in_file(self, mock_stdout):
        year = "2021"
        license_id = "AGPL-3.0-or-later"

        header = HEADER.format(date="2020")
        with temp_file(
            content=header, name="test.py", change_into=True
        ) as test_file:
            update_file(
                test_file,
                year,
                license_id,
                self.company,
            )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                ret,
                f"{test_file}: Changed License Header "
                "Copyright Year None -> 2021\n",
            )
            self.assertIn(
                "# SPDX-FileCopyrightText: 2020-2021 Greenbone AG",
                test_file.read_text(encoding="utf-8"),
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_header_in_file_single_year(self, mock_stdout):
        year = "2021"
        license_id = "AGPL-3.0-or-later"

        header = HEADER.format(date="2020-2021")
        with temp_file(
            content=header, name="test.py", change_into=True
        ) as test_file:
            update_file(
                test_file,
                year,
                license_id,
                self.company,
                single_year=True,
            )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                ret,
                f"{test_file}: Changed License Header Copyright Year format to single year "
                "2020-2021 -> 2020\n",
            )

            self.assertIn(
                "# SPDX-FileCopyrightText: 2020 Greenbone AG",
                test_file.read_text(encoding="utf-8"),
            )

    @patch("sys.stdout", new_callable=StringIO)
    def test_update_header_ok_in_file(self, mock_stdout):
        year = "2021"
        license_id = "AGPL-3.0-or-later"

        header = HEADER.format(date="2021")
        with temp_file(
            content=header, name="test.py", change_into=True
        ) as test_file:
            update_file(
                test_file,
                year,
                license_id,
                self.company,
            )

            ret = mock_stdout.getvalue()
            self.assertEqual(
                ret,
                f"{test_file}: License Header is ok.\n",
            )
            self.assertIn(
                "# SPDX-FileCopyrightText: 2021 Greenbone AG",
                test_file.read_text(encoding="utf-8"),
            )

    def test_cleanup_file(self):
        test_content = """# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import foo
import bar

foo.baz(bar.boing)
"""  # noqa: E501

        expected_content = f"""# SPDX-FileCopyrightText: 2021-{str(datetime.datetime.now().year)} Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import foo
import bar

foo.baz(bar.boing)
"""  # noqa: E501

        company = "Greenbone AG"
        year = str(datetime.datetime.now().year)
        license_id = "GPL-3.0-or-later"

        with temp_file(content=test_content, name="foo.py") as tmp:

            update_file(
                tmp,
                year,
                license_id,
                company,
                cleanup=True,
            )

            new_content = tmp.read_text(encoding="utf-8")
            self.assertEqual(expected_content, new_content)

    def test_cleanup_file_spdx_header(self):
        test_content = """
# SPDX-FileCopyrightText: 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import foo
import bar

foo.baz(bar.boing)
"""  # noqa: E501

        expected_content = f"""
# SPDX-FileCopyrightText: 2021-{str(datetime.datetime.now().year)} Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import foo
import bar

foo.baz(bar.boing)
"""  # noqa: E501

        company = "Greenbone AG"
        year = str(datetime.datetime.now().year)
        license_id = "GPL-3.0-or-later"

        with temp_file(content=test_content, name="foo.py") as tmp:

            update_file(
                tmp,
                year,
                license_id,
                company,
                cleanup=True,
            )

            new_content = tmp.read_text(encoding="utf-8")
            self.assertEqual(expected_content, new_content)

    def test_cleanup_file_changed_company(self):
        test_content = """
# SPDX-FileCopyrightText: 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import foo
import bar

foo.baz(bar.boing)
"""  # noqa: E501

        expected_content = f"""
# SPDX-FileCopyrightText: 2021-{str(datetime.datetime.now().year)} ACME Inc.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import foo
import bar

foo.baz(bar.boing)
"""  # noqa: E501

        company = "ACME Inc."
        year = str(datetime.datetime.now().year)
        license_id = "GPL-3.0-or-later"

        with temp_file(content=test_content, name="foo.py") as tmp:

            update_file(
                tmp,
                year,
                license_id,
                company,
                cleanup=True,
            )

            new_content = tmp.read_text(encoding="utf-8")
            self.assertEqual(expected_content, new_content)


class ParseArgsTestCase(TestCase):
    def test_argparser_files(self):
        args = ["-f", "test.py", "-y", "2021", "-l", "AGPL-3.0-or-later"]
        args = parse_args(args)

        self.assertEqual(args.company, "Greenbone AG")
        self.assertEqual(args.files, ["test.py"])
        self.assertEqual(args.year, "2021")
        self.assertEqual(args.license_id, "AGPL-3.0-or-later")

    def test_argparser_dir(self):
        args = ["-d", ".", "-c", "-l", "AGPL-3.0-or-later"]
        args = parse_args(args)

        self.assertEqual(args.directories, ["."])
        self.assertEqual(args.company, "Greenbone AG")
        self.assertTrue(args.changed)
        self.assertEqual(args.year, str(datetime.datetime.now().year))
        self.assertEqual(args.license_id, "AGPL-3.0-or-later")

    def test_defaults(self):
        args = ["-f", "foo.txt"]
        args = parse_args(args)

        self.assertFalse(args.quiet)
        self.assertIsNone(args.log_file)
        self.assertFalse(args.changed)
        self.assertEqual(args.year, str(datetime.date.today().year))
        self.assertEqual(args.license_id, "GPL-3.0-or-later")
        self.assertEqual(args.company, "Greenbone AG")
        self.assertEqual(args.files, ["foo.txt"])
        self.assertIsNone(args.directories)
        self.assertIsNone(args.exclude_file)
        self.assertFalse(args.single_year)
        self.assertFalse(args.cleanup)

    def test_files_and_directories_mutual_exclusive(self):
        args = ["--files", "foo", "--directories", "bar"]
        with self.assertRaises(SystemExit) as cm:
            args = parse_args(args)

            self.assertIn(
                "argument -d/--directories: not allowed with argument -f/--file",
                cm.msg,
            )


class GetExcludeListTestCase(TestCase):
    def test_get_exclude_list(self):
        # Try to find the current file from two directories up...
        test_dirname = Path(__file__).parent.parent.parent
        # with a relative glob
        test_ignore_file = Path("ignore.file")
        test_ignore_file.write_text("*.py\n", encoding="utf-8")

        exclude_list = get_exclude_list(
            test_ignore_file, [test_dirname.resolve()]
        )

        self.assertIn(Path(__file__), exclude_list)

        test_ignore_file.unlink()


class MainTestCase(TestCase):
    def setUp(self) -> None:
        self.args = Namespace()
        self.args.company = "Greenbone AG"

    def test_main(self):
        args = [
            "--year",
            "2021",
            "--license",
            "AGPL-3.0-or-later",
            "--files",
            "test.py",
        ]
        with redirect_stdout(StringIO()):
            main(args)

    @patch("sys.stdout", new_callable=StringIO)
    @patch("pontos.updateheader.updateheader.parse_args")
    def test_main_never_happen(self, argparser_mock, mock_stdout):
        self.args.year = "2021"
        self.args.changed = False
        self.args.license_id = "AGPL-3.0-or-later"
        self.args.files = None
        self.args.directories = None
        self.args.verbose = 0
        self.args.log_file = None
        self.args.quiet = False
        self.args.cleanup = False
        self.args.single_year = False

        argparser_mock.return_value = self.args

        # I have no idea how or why test main ...
        with self.assertRaises(SystemExit):
            main()

        ret = mock_stdout.getvalue()
        self.assertIn(
            "Specify files to update!",
            ret,
        )

    def test_update_file_changed_no_git(self):
        args = [
            "--changed",
            "--year",
            "1999",
            "--files",
        ]

        with (
            redirect_stdout(StringIO()) as out,
            temp_directory(change_into=True) as temp_dir,
        ):
            test_file = temp_dir / "test.py"
            args.append(str(test_file))

            main(args)

            ret = out.getvalue()

            self.assertIn(
                "Could not get date of last modification via git, "
                f"using 1999 instead.{test_file}: File is not existing.",
                ret.replace("\n", ""),
            )


class RemoveOutdatedLinesTestCase(TestCase):
    def setUp(self) -> None:
        self.compiled_regexes = compile_outdated_regex()

    def test_remove_outdated_lines(self):
        test_content = """* This program is free software: you can redistribute it and/or modify
*it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the
//License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
# modify it under the terms of the GNU General Public License
# This program is free software; you can redistribute it and/or
# version 2 as published by the Free Software Foundation.
This program is free software: you can redistribute it and/or modify"""  # noqa: E501

        new_content = remove_outdated_lines(
            content=test_content, cleanup_regexes=self.compiled_regexes
        )
        self.assertEqual(new_content, "\n")

    def test_remove_outdated_lines2(self):
        test_content = """the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.
* GNU General Public License for more details.
You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA."""  # noqa: E501

        new_content = remove_outdated_lines(
            content=test_content, cleanup_regexes=self.compiled_regexes
        )
        self.assertEqual(new_content, "\n")



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=line-too-long

from unittest.mock import MagicMock

import httpx

from pontos.github.api.repositories import (
    GitHubAsyncRESTRepositories,
    GitIgnoreTemplate,
    LicenseType,
    MergeCommitMessage,
    MergeCommitTitle,
    SquashMergeCommitMessage,
    SquashMergeCommitTitle,
)
from tests.github.api import GitHubAsyncRESTTestCase, create_response

REPOSITORY_JSON = {
    "id": 1,
    "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
    "name": "Hello-World",
    "full_name": "octocat/Hello-World",
    "owner": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "private": False,
    "html_url": "https://github.com/octocat/Hello-World",
    "description": "This your first repo!",
    "fork": False,
    "url": "https://api.github.com/repos/octocat/Hello-World",
    "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
    "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
    "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
    "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
    "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
    "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
    "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
    "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
    "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
    "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
    "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
    "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
    "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
    "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
    "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
    "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
    "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
    "git_url": "git:github.com/octocat/Hello-World.git",
    "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
    "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
    "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
    "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
    "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
    "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
    "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
    "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
    "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
    "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
    "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
    "ssh_url": "git@github.com:octocat/Hello-World.git",
    "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
    "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
    "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
    "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
    "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
    "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
    "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
    "clone_url": "https://github.com/octocat/Hello-World.git",
    "mirror_url": "git:git.example.com/octocat/Hello-World",
    "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
    "svn_url": "https://svn.github.com/octocat/Hello-World",
    "homepage": "https://github.com",
    "language": None,
    "forks_count": 9,
    "forks": 9,
    "stargazers_count": 80,
    "watchers_count": 80,
    "watchers": 80,
    "size": 108,
    "default_branch": "master",
    "open_issues_count": 0,
    "open_issues": 0,
    "is_template": False,
    "topics": ["octocat", "atom", "electron", "api"],
    "has_issues": True,
    "has_projects": True,
    "has_wiki": True,
    "has_pages": False,
    "has_downloads": True,
    "has_discussions": False,
    "archived": False,
    "disabled": False,
    "visibility": "public",
    "pushed_at": "2011-01-26T19:06:43Z",
    "created_at": "2011-01-26T19:01:12Z",
    "updated_at": "2011-01-26T19:14:43Z",
    "permissions": {"pull": True, "push": False, "admin": False},
    "allow_rebase_merge": True,
    "template_repository": {
        "id": 1296269,
        "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
        "name": "Hello-World-Template",
        "full_name": "octocat/Hello-World-Template",
        "owner": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "private": False,
        "html_url": "https://github.com/octocat/Hello-World-Template",
        "description": "This your first repo!",
        "fork": False,
        "url": "https://api.github.com/repos/octocat/Hello-World-Template",
        "archive_url": "https://api.github.com/repos/octocat/Hello-World-Template/{archive_format}{/ref}",
        "assignees_url": "https://api.github.com/repos/octocat/Hello-World-Template/assignees{/user}",
        "blobs_url": "https://api.github.com/repos/octocat/Hello-World-Template/git/blobs{/sha}",
        "branches_url": "https://api.github.com/repos/octocat/Hello-World-Template/branches{/branch}",
        "collaborators_url": "https://api.github.com/repos/octocat/Hello-World-Template/collaborators{/collaborator}",
        "comments_url": "https://api.github.com/repos/octocat/Hello-World-Template/comments{/number}",
        "commits_url": "https://api.github.com/repos/octocat/Hello-World-Template/commits{/sha}",
        "compare_url": "https://api.github.com/repos/octocat/Hello-World-Template/compare/{base}...{head}",
        "contents_url": "https://api.github.com/repos/octocat/Hello-World-Template/contents/{+path}",
        "contributors_url": "https://api.github.com/repos/octocat/Hello-World-Template/contributors",
        "deployments_url": "https://api.github.com/repos/octocat/Hello-World-Template/deployments",
        "downloads_url": "https://api.github.com/repos/octocat/Hello-World-Template/downloads",
        "events_url": "https://api.github.com/repos/octocat/Hello-World-Template/events",
        "forks_url": "https://api.github.com/repos/octocat/Hello-World-Template/forks",
        "git_commits_url": "https://api.github.com/repos/octocat/Hello-World-Template/git/commits{/sha}",
        "git_refs_url": "https://api.github.com/repos/octocat/Hello-World-Template/git/refs{/sha}",
        "git_tags_url": "https://api.github.com/repos/octocat/Hello-World-Template/git/tags{/sha}",
        "git_url": "git:github.com/octocat/Hello-World-Template.git",
        "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World-Template/issues/comments{/number}",
        "issue_events_url": "https://api.github.com/repos/octocat/Hello-World-Template/issues/events{/number}",
        "issues_url": "https://api.github.com/repos/octocat/Hello-World-Template/issues{/number}",
        "keys_url": "https://api.github.com/repos/octocat/Hello-World-Template/keys{/key_id}",
        "labels_url": "https://api.github.com/repos/octocat/Hello-World-Template/labels{/name}",
        "languages_url": "https://api.github.com/repos/octocat/Hello-World-Template/languages",
        "merges_url": "https://api.github.com/repos/octocat/Hello-World-Template/merges",
        "milestones_url": "https://api.github.com/repos/octocat/Hello-World-Template/milestones{/number}",
        "notifications_url": "https://api.github.com/repos/octocat/Hello-World-Template/notifications{?since,all,participating}",
        "pulls_url": "https://api.github.com/repos/octocat/Hello-World-Template/pulls{/number}",
        "releases_url": "https://api.github.com/repos/octocat/Hello-World-Template/releases{/id}",
        "ssh_url": "git@github.com:octocat/Hello-World-Template.git",
        "stargazers_url": "https://api.github.com/repos/octocat/Hello-World-Template/stargazers",
        "statuses_url": "https://api.github.com/repos/octocat/Hello-World-Template/statuses/{sha}",
        "subscribers_url": "https://api.github.com/repos/octocat/Hello-World-Template/subscribers",
        "subscription_url": "https://api.github.com/repos/octocat/Hello-World-Template/subscription",
        "tags_url": "https://api.github.com/repos/octocat/Hello-World-Template/tags",
        "teams_url": "https://api.github.com/repos/octocat/Hello-World-Template/teams",
        "trees_url": "https://api.github.com/repos/octocat/Hello-World-Template/git/trees{/sha}",
        "clone_url": "https://github.com/octocat/Hello-World-Template.git",
        "mirror_url": "git:git.example.com/octocat/Hello-World-Template",
        "hooks_url": "https://api.github.com/repos/octocat/Hello-World-Template/hooks",
        "svn_url": "https://svn.github.com/octocat/Hello-World-Template",
        "homepage": "https://github.com",
        "language": None,
        "forks": 9,
        "forks_count": 9,
        "stargazers_count": 80,
        "watchers_count": 80,
        "watchers": 80,
        "size": 108,
        "default_branch": "master",
        "open_issues": 0,
        "open_issues_count": 0,
        "is_template": True,
        "license": {
            "key": "mit",
            "name": "MIT License",
            "url": "https://api.github.com/licenses/mit",
            "spdx_id": "MIT",
            "node_id": "MDc6TGljZW5zZW1pdA==",
            "html_url": "https://api.github.com/licenses/mit",
        },
        "topics": ["octocat", "atom", "electron", "api"],
        "has_issues": True,
        "has_projects": True,
        "has_wiki": True,
        "has_pages": False,
        "has_downloads": True,
        "archived": False,
        "disabled": False,
        "visibility": "public",
        "pushed_at": "2011-01-26T19:06:43Z",
        "created_at": "2011-01-26T19:01:12Z",
        "updated_at": "2011-01-26T19:14:43Z",
        "permissions": {"admin": False, "push": False, "pull": True},
        "allow_rebase_merge": True,
        "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
        "allow_squash_merge": True,
        "allow_auto_merge": False,
        "delete_branch_on_merge": True,
        "allow_merge_commit": True,
        "subscribers_count": 42,
        "network_count": 0,
    },
    "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
    "allow_squash_merge": True,
    "allow_auto_merge": False,
    "delete_branch_on_merge": True,
    "allow_merge_commit": True,
    "subscribers_count": 42,
    "network_count": 0,
    "license": {
        "key": "mit",
        "name": "MIT License",
        "spdx_id": "MIT",
        "url": "https://api.github.com/licenses/mit",
        "node_id": "MDc6TGljZW5zZW1pdA==",
    },
    "organization": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "Organization",
        "site_admin": False,
    },
    "parent": {
        "id": 1296269,
        "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
        "name": "Hello-World",
        "full_name": "octocat/Hello-World",
        "owner": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "private": False,
        "html_url": "https://github.com/octocat/Hello-World",
        "description": "This your first repo!",
        "fork": False,
        "url": "https://api.github.com/repos/octocat/Hello-World",
        "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
        "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
        "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
        "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
        "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
        "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
        "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
        "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
        "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
        "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
        "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
        "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
        "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
        "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
        "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
        "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
        "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
        "git_url": "git:github.com/octocat/Hello-World.git",
        "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
        "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
        "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
        "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
        "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
        "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
        "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
        "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
        "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
        "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
        "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
        "ssh_url": "git@github.com:octocat/Hello-World.git",
        "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
        "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
        "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
        "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
        "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
        "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
        "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
        "clone_url": "https://github.com/octocat/Hello-World.git",
        "mirror_url": "git:git.example.com/octocat/Hello-World",
        "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
        "svn_url": "https://svn.github.com/octocat/Hello-World",
        "homepage": "https://github.com",
        "language": None,
        "forks_count": 9,
        "stargazers_count": 80,
        "watchers_count": 80,
        "size": 108,
        "default_branch": "master",
        "open_issues_count": 0,
        "is_template": True,
        "topics": ["octocat", "atom", "electron", "api"],
        "has_issues": True,
        "has_projects": True,
        "has_wiki": True,
        "has_pages": False,
        "has_downloads": True,
        "archived": False,
        "disabled": False,
        "visibility": "public",
        "pushed_at": "2011-01-26T19:06:43Z",
        "created_at": "2011-01-26T19:01:12Z",
        "updated_at": "2011-01-26T19:14:43Z",
        "permissions": {"admin": False, "push": False, "pull": True},
        "allow_rebase_merge": True,
        "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
        "allow_squash_merge": True,
        "allow_auto_merge": False,
        "delete_branch_on_merge": True,
        "allow_merge_commit": True,
        "subscribers_count": 42,
        "network_count": 0,
        "license": {
            "key": "mit",
            "name": "MIT License",
            "url": "https://api.github.com/licenses/mit",
            "spdx_id": "MIT",
            "node_id": "MDc6TGljZW5zZW1pdA==",
            "html_url": "https://api.github.com/licenses/mit",
        },
        "forks": 1,
        "open_issues": 1,
        "watchers": 1,
    },
    "source": {
        "id": 1296269,
        "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
        "name": "Hello-World",
        "full_name": "octocat/Hello-World",
        "owner": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "private": False,
        "html_url": "https://github.com/octocat/Hello-World",
        "description": "This your first repo!",
        "fork": False,
        "url": "https://api.github.com/repos/octocat/Hello-World",
        "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
        "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
        "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
        "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
        "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
        "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
        "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
        "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
        "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
        "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
        "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
        "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
        "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
        "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
        "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
        "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
        "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
        "git_url": "git:github.com/octocat/Hello-World.git",
        "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
        "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
        "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
        "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
        "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
        "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
        "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
        "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
        "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
        "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
        "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
        "ssh_url": "git@github.com:octocat/Hello-World.git",
        "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
        "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
        "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
        "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
        "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
        "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
        "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
        "clone_url": "https://github.com/octocat/Hello-World.git",
        "mirror_url": "git:git.example.com/octocat/Hello-World",
        "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
        "svn_url": "https://svn.github.com/octocat/Hello-World",
        "homepage": "https://github.com",
        "language": None,
        "forks_count": 9,
        "stargazers_count": 80,
        "watchers_count": 80,
        "size": 108,
        "default_branch": "master",
        "open_issues_count": 0,
        "is_template": True,
        "topics": ["octocat", "atom", "electron", "api"],
        "has_issues": True,
        "has_projects": True,
        "has_wiki": True,
        "has_pages": False,
        "has_downloads": True,
        "archived": False,
        "disabled": False,
        "visibility": "public",
        "pushed_at": "2011-01-26T19:06:43Z",
        "created_at": "2011-01-26T19:01:12Z",
        "updated_at": "2011-01-26T19:14:43Z",
        "permissions": {"admin": False, "push": False, "pull": True},
        "allow_rebase_merge": True,
        "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
        "allow_squash_merge": True,
        "allow_auto_merge": False,
        "delete_branch_on_merge": True,
        "allow_merge_commit": True,
        "subscribers_count": 42,
        "network_count": 0,
        "license": {
            "key": "mit",
            "name": "MIT License",
            "url": "https://api.github.com/licenses/mit",
            "spdx_id": "MIT",
            "node_id": "MDc6TGljZW5zZW1pdA==",
            "html_url": "https://api.github.com/licenses/mit",
        },
        "forks": 1,
        "open_issues": 1,
        "watchers": 1,
    },
}


class GitHubAsyncRESTRepositoriesTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTRepositories

    async def test_get(self):
        response = create_response()
        response.json.return_value = REPOSITORY_JSON
        self.client.get.return_value = response

        repo = await self.api.get("foo/bar")

        self.client.get.assert_awaited_once_with("/repos/foo/bar")

        self.assertEqual(repo.id, 1)

    async def test_get_failure(self):
        response = create_response()
        self.client.get.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.get("foo/bar")

        self.client.get.assert_awaited_once_with("/repos/foo/bar")

    async def test_delete(self):
        response = create_response()
        self.client.delete.return_value = response

        await self.api.delete("foo/bar")

        self.client.delete.assert_awaited_once_with("/repos/foo/bar")

    async def test_delete_failure(self):
        response = create_response()
        self.client.delete.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.delete("foo/bar")

        self.client.delete.assert_awaited_once_with("/repos/foo/bar")

    async def test_create_with_defaults(self):
        response = create_response()
        response.json.return_value = REPOSITORY_JSON
        self.client.post.return_value = response

        repo = await self.api.create("foo", "bar")

        self.client.post.assert_awaited_once_with(
            "/orgs/foo/repos",
            data={
                "name": "bar",
                "private": False,
                "has_issues": True,
                "has_projects": True,
                "has_wiki": True,
                "is_template": False,
                "has_downloads": True,
                "auto_init": False,
                "allow_squash_merge": True,
                "allow_merge_commit": True,
                "allow_rebase_merge": True,
                "allow_auto_merge": False,
                "allow_update_branch": False,
                "delete_branch_on_merge": False,
            },
        )

        self.assertEqual(repo.id, 1)

    async def test_create(self):
        response = create_response()
        response.json.return_value = REPOSITORY_JSON
        self.client.post.return_value = response

        repo = await self.api.create(
            "foo",
            "bar",
            private=True,
            has_issues=False,
            has_projects=False,
            has_wiki=False,
            is_template=True,
            team_id="123",
            has_downloads=False,
            auto_init=True,
            gitignore_template=GitIgnoreTemplate.PYTHON,
            license_template=LicenseType.MIT,
            allow_squash_merge=False,
            allow_merge_commit=False,
            allow_rebase_merge=False,
            allow_auto_merge=True,
            allow_update_branch=True,
            delete_branch_on_merge=True,
            squash_merge_commit_title=SquashMergeCommitTitle.COMMIT_OR_PR_TITLE,
            squash_merge_commit_message=SquashMergeCommitMessage.PR_BODY,
            merge_commit_title=MergeCommitTitle.MERGE_MESSAGE,
            merge_commit_message=MergeCommitMessage.PR_BODY,
        )

        self.client.post.assert_awaited_once_with(
            "/orgs/foo/repos",
            data={
                "name": "bar",
                "private": True,
                "has_issues": False,
                "has_projects": False,
                "has_wiki": False,
                "is_template": True,
                "team_id": "123",
                "has_downloads": False,
                "auto_init": True,
                "license_template": "mit",
                "gitignore_template": "Python",
                "allow_squash_merge": False,
                "allow_merge_commit": False,
                "allow_rebase_merge": False,
                "allow_auto_merge": True,
                "allow_update_branch": True,
                "delete_branch_on_merge": True,
                "squash_merge_commit_title": "COMMIT_OR_PR_TITLE",
                "squash_merge_commit_message": "PR_BODY",
                "merge_commit_title": "MERGE_MESSAGE",
                "merge_commit_message": "PR_BODY",
            },
        )

        self.assertEqual(repo.id, 1)

    async def test_archive(self):
        response = create_response()
        self.client.post.return_value = response

        await self.api.archive("foo/bar")

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar", data={"archived": True}
        )

    async def test_archive_failure(self):
        response = create_response()
        self.client.post.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.archive("foo/bar")

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar", data={"archived": True}
        )

    async def test_update(self):
        response = create_response()
        response.json.return_value = REPOSITORY_JSON
        self.client.post.return_value = response

        repo = await self.api.update(
            "foo/bar",
            name="baz",
            description="A new Baz",
            homepage="http://baz.com",
            private=True,
            has_issues=False,
            has_projects=False,
            has_wiki=False,
            is_template=True,
            allow_squash_merge=False,
            allow_merge_commit=False,
            allow_rebase_merge=False,
            allow_auto_merge=True,
            allow_update_branch=True,
            delete_branch_on_merge=True,
            squash_merge_commit_title=SquashMergeCommitTitle.PR_TITLE,
            squash_merge_commit_message=SquashMergeCommitMessage.PR_BODY,
            merge_commit_title=MergeCommitTitle.PR_TITLE,
            merge_commit_message=MergeCommitMessage.PR_BODY,
            allow_forking=True,
            web_commit_signoff_required=True,
        )

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar",
            data={
                "name": "baz",
                "description": "A new Baz",
                "homepage": "http://baz.com",
                "private": True,
                "has_issues": False,
                "has_projects": False,
                "has_wiki": False,
                "is_template": True,
                "allow_squash_merge": False,
                "allow_merge_commit": False,
                "allow_rebase_merge": False,
                "allow_auto_merge": True,
                "allow_update_branch": True,
                "delete_branch_on_merge": True,
                "squash_merge_commit_title": "PR_TITLE",
                "squash_merge_commit_message": "PR_BODY",
                "merge_commit_title": "PR_TITLE",
                "merge_commit_message": "PR_BODY",
                "allow_forking": True,
                "web_commit_signoff_required": True,
            },
        )

        self.assertEqual(repo.id, 1)

    async def test_update_with_defaults(self):
        response = create_response()
        response.json.return_value = REPOSITORY_JSON
        self.client.post.return_value = response

        repo = await self.api.update("foo/bar")

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar",
            data={
                "private": False,
                "has_issues": True,
                "has_projects": True,
                "has_wiki": True,
                "is_template": False,
                "allow_squash_merge": True,
                "allow_merge_commit": True,
                "allow_rebase_merge": True,
                "allow_auto_merge": False,
                "allow_update_branch": False,
                "delete_branch_on_merge": False,
                "allow_forking": False,
                "web_commit_signoff_required": False,
            },
        )

        self.assertEqual(repo.id, 1)

    async def test_update_failure(self):
        response = create_response()
        self.client.post.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.update("foo/bar")

        self.client.post.assert_awaited_once_with(
            "/repos/foo/bar",
            data={
                "private": False,
                "has_issues": True,
                "has_projects": True,
                "has_wiki": True,
                "is_template": False,
                "allow_squash_merge": True,
                "allow_merge_commit": True,
                "allow_rebase_merge": True,
                "allow_auto_merge": False,
                "allow_update_branch": False,
                "delete_branch_on_merge": False,
                "allow_forking": False,
                "web_commit_signoff_required": False,
            },
        )

    async def test_get_topics(self):
        response = create_response()
        response.json.return_value = {"names": ["foo", "bar", "baz"]}
        self.client.get.return_value = response

        topics = await self.api.topics("foo/bar")

        self.client.get.assert_awaited_once_with("/repos/foo/bar/topics")

        self.assertEqual(len(topics), 3)
        self.assertEqual(topics, ["foo", "bar", "baz"])

    async def test_update_topics(self):
        response = create_response()
        response.json.return_value = {}
        response.json.return_value = {"names": ["foo", "bar"]}

        self.client.put.return_value = response

        new_topics = await self.api.update_topics(
            "foo/bar",
            (
                "foo",
                "bar",
            ),
        )

        self.client.put.assert_awaited_once_with(
            "/repos/foo/bar/topics", data={"names": ["foo", "bar"]}
        )

        self.assertEqual(len(new_topics), 2)
        self.assertEqual(new_topics, ["foo", "bar"])



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys
from enum import IntEnum, auto
from typing import List, NoReturn, Optional

from pontos.errors import PontosError

from ._parser import parse_args
from .project import Project
from .schemes import VersioningScheme


class VersionExitCode(IntEnum):
    SUCCESS = 0
    NO_PROJECT = auto()
    UPDATE_ERROR = auto()
    CURRENT_VERSION_ERROR = auto()
    VERIFY_ERROR = auto()
    NEXT_VERSION_ERROR = auto()


def main(args: Optional[List[str]] = None) -> NoReturn:
    parsed_args = parse_args(args)

    try:
        project = Project(parsed_args.versioning_scheme)
    except PontosError:
        print("No project found.", file=sys.stderr)
        sys.exit(VersionExitCode.NO_PROJECT)

    if parsed_args.command == "update":
        try:
            update = project.update_version(
                parsed_args.version, force=parsed_args.force
            )
        except PontosError as e:
            print(str(e), file=sys.stderr)
            sys.exit(VersionExitCode.UPDATE_ERROR)

        if update.new == update.previous:
            print("Version is already up-to-date.")
        else:
            print(f"Updated version from {update.previous} to {update.new}.")
    elif parsed_args.command == "show":
        try:
            print(str(project.get_current_version()))
        except PontosError as e:
            print(str(e), file=sys.stderr)
            sys.exit(VersionExitCode.CURRENT_VERSION_ERROR)
    elif parsed_args.command == "verify":
        try:
            project.verify_version(parsed_args.version)
        except PontosError as e:
            print(str(e), file=sys.stderr)
            sys.exit(VersionExitCode.VERIFY_ERROR)
    elif parsed_args.command == "next":
        scheme: VersioningScheme = parsed_args.versioning_scheme
        calculator = scheme.calculator()
        try:
            current_version = project.get_current_version()
        except PontosError as e:
            print(str(e), file=sys.stderr)
            sys.exit(VersionExitCode.CURRENT_VERSION_ERROR)

        if parsed_args.type == "dev":
            print(calculator.next_dev_version(current_version))
        elif parsed_args.type == "calendar":
            print(calculator.next_calendar_version(current_version))
        elif parsed_args.type == "alpha":
            print(calculator.next_alpha_version(current_version))
        elif parsed_args.type == "beta":
            print(calculator.next_beta_version(current_version))
        elif parsed_args.type == "rc":
            print(calculator.next_release_candidate_version(current_version))
        elif parsed_args.type == "patch":
            print(calculator.next_patch_version(current_version))
        elif parsed_args.type == "minor":
            print(calculator.next_minor_version(current_version))
        elif parsed_args.type == "major":
            print(calculator.next_major_version(current_version))

    sys.exit(VersionExitCode.SUCCESS)



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
import unittest
from pathlib import Path
from string import Template

from pontos.testing import temp_directory, temp_file
from pontos.version import VersionError
from pontos.version.commands import JavaVersionCommand
from pontos.version.schemes import SemanticVersioningScheme

TEMPLATE_UPGRADE_VERSION_SINGLE_JSON = """{
  "files": [
    {
      "path": "README.md",
      "line": 3
    }
  ]
}
"""

TEMPLATE_UPGRADE_VERSION_MULTI_JSON = """{
  "files": [
    {
      "path": "README.md",
      "line": 3
    },
    {
      "path": "application.properties",
      "line": 2
    }
  ]
}
"""

TEMPLATE_UPGRADE_VERSION_WITH_LINE_JSON = Template(
    """{
  "files": [
    {
      "path": "README.md",
      "line": ${LINE_NO}
    }
  ]
}
"""
)

TEMPLATE_UPGRADE_VERSION_MARKDOWN = """# Task service

**task service**: Version {}

## starting the local 
"""

TEMPLATE_UPGRADE_VERSION_WITH_VERSION_PROPERTIES = """# application
sentry.release={}
server.port=8080
"""


class VerifyJavaVersionParsingTestCase(unittest.TestCase):
    def test_version_parsing(self):
        versions = {
            "2023.12.10",
            "2023.1.1",
            "2023.10.1",
            "2023.1.99",
            "0.0.1",
            "1.2.3-a1",
            "1.2.3-alpha1",
            "1.2.3-alpha1-dev1",
            "1.2.3-b1",
            "1.2.3-beta1",
            "1.2.3-beta1-dev1",
            "1.2.3-rc1",
            "1.2.3-rc1-dev1",
            "1.2.3-dev1",
            "22.4.1",
            "22.4.1-dev1",
            "0.5.0.dev1",
            "1.0.0-dev1",
            "1.0.0-alpha1",
            "1.0.0-alpha1-dev1",
            "1.0.0-beta1",
            "1.0.0-beta1-dev1",
            "1.0.0-rc1",
            "1.0.0-rc1-dev1",
        }
        for version in versions:
            with self.subTest(version=version):
                matches = JavaVersionCommand(
                    SemanticVersioningScheme
                ).parse_line(f"pre{version}post")

                self.assertEqual(matches.group("pre"), "pre")
                self.assertEqual(matches.group("version"), version)
                self.assertEqual(matches.group("post"), "post")


class GetCurrentJavaVersionCommandTestCase(unittest.TestCase):
    def test_getting_version(self):
        with temp_directory(change_into=True):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_SINGLE_JSON,
                encoding="utf-8",
            )

            version = "2023.9.3"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )

            result_version = JavaVersionCommand(
                SemanticVersioningScheme
            ).get_current_version()

            self.assertEqual(
                result_version, SemanticVersioningScheme.parse_version(version)
            )

            version_file_path.unlink()
            readme_file_path.unlink()

    def test_getting_version_no_files_configured(self):
        exp_err_msg = "no version found"
        with (
            temp_directory(change_into=True),
            self.assertRaisesRegex(
                VersionError,
                exp_err_msg,
            ),
        ):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                """{"files": []}""",
                encoding="utf-8",
            )

            JavaVersionCommand(SemanticVersioningScheme).get_current_version()

            version_file_path.unlink()

    def test_getting_version_without_version_config(self):
        exp_err_msg = (
            r"No /tmp/.*/upgradeVersion\.json config file found\. "
            r"This file is required for pontos"
        )
        with (
            temp_directory(change_into=True),
            self.assertRaisesRegex(
                VersionError,
                exp_err_msg,
            ),
        ):
            JavaVersionCommand(SemanticVersioningScheme).get_current_version()


class VerifyJavaVersionCommandTestCase(unittest.TestCase):
    def test_verify_version(self):
        with temp_directory(change_into=True):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MULTI_JSON, encoding="utf-8"
            )

            version = "2023.9.3"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )
            properties_file_path = Path("application.properties")
            properties_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_WITH_VERSION_PROPERTIES.format(
                    version
                ),
                encoding="latin-1",
            )

            JavaVersionCommand(SemanticVersioningScheme).verify_version(
                SemanticVersioningScheme.parse_version(version)
            )

            version_file_path.unlink()
            readme_file_path.unlink()
            properties_file_path.unlink()

    def test_verify_version_does_not_match(self):
        exp_err_msg = (
            r"Provided version 2023\.9\.4 does not match the "
            + r"current version 2023\.9\.3 "
            + r"in '/tmp/.*/upgradeVersion\.json'"
        )

        with (
            temp_directory(change_into=True),
            self.assertRaisesRegex(
                VersionError,
                exp_err_msg,
            ),
        ):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_SINGLE_JSON, encoding="utf-8"
            )

            version = "2023.9.3"
            new_version = "2023.9.4"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )

            JavaVersionCommand(SemanticVersioningScheme).verify_version(
                SemanticVersioningScheme.parse_version(new_version)
            )

            version_file_path.unlink()
            readme_file_path.unlink()


class UpdateJavaVersionCommandTestCase(unittest.TestCase):
    def test_update_version(self):
        with temp_directory(change_into=True):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_SINGLE_JSON, encoding="utf-8"
            )

            version = "2023.9.3"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )

            new_version = "2023.9.4"
            updated_version_obj = JavaVersionCommand(
                SemanticVersioningScheme
            ).update_version(
                SemanticVersioningScheme.parse_version(new_version)
            )

            self.assertEqual(
                updated_version_obj.previous,
                SemanticVersioningScheme.parse_version(version),
            )
            self.assertEqual(
                updated_version_obj.new,
                SemanticVersioningScheme.parse_version(new_version),
            )
            self.assertEqual(
                updated_version_obj.changed_files, [Path("README.md")]
            )

            content = readme_file_path.read_text(encoding="UTF-8")
            self.assertEqual(
                content,
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(new_version),
            )

            version_file_path.unlink()
            readme_file_path.unlink()

    def test_no_update_version(self):
        with temp_directory(change_into=True):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_SINGLE_JSON,
                encoding="utf-8",
            )

            version = "2023.9.3"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )

            updated_version_obj = JavaVersionCommand(
                SemanticVersioningScheme
            ).update_version(SemanticVersioningScheme.parse_version(version))

            self.assertEqual(
                updated_version_obj.previous,
                SemanticVersioningScheme.parse_version(version),
            )
            self.assertEqual(
                updated_version_obj.new,
                SemanticVersioningScheme.parse_version(version),
            )
            self.assertEqual(
                updated_version_obj.changed_files,
                [],
            )

            content = readme_file_path.read_text(encoding="UTF-8")
            self.assertEqual(
                content,
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
            )

            version_file_path.unlink()
            readme_file_path.unlink()

    def test_forced_update_version(self):
        with temp_directory(change_into=True):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_SINGLE_JSON,
                encoding="utf-8",
            )

            version = "2023.9.3"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )

            updated_version_obj = JavaVersionCommand(
                SemanticVersioningScheme
            ).update_version(
                SemanticVersioningScheme.parse_version(version), force=True
            )

            self.assertEqual(
                updated_version_obj.previous,
                SemanticVersioningScheme.parse_version(version),
            )
            self.assertEqual(
                updated_version_obj.new,
                SemanticVersioningScheme.parse_version(version),
            )
            self.assertEqual(
                updated_version_obj.changed_files,
                [Path("README.md")],
            )

            content = readme_file_path.read_text(encoding="UTF-8")
            self.assertEqual(
                content,
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
            )

            version_file_path.unlink()
            readme_file_path.unlink()

    def test_update_version_upgrade_config_with_wrong_line_number(self):
        exp_err_msg = (
            "Line has no version, "
            "file:'README.md' "
            "lineNo:4 "
            "content:'\n'"
        )
        with (
            temp_directory(change_into=True),
            self.assertRaisesRegex(
                VersionError,
                exp_err_msg,
            ),
        ):
            version_file_path = Path("upgradeVersion.json")
            version_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_WITH_LINE_JSON.substitute(LINE_NO="4"),
                encoding="utf-8",
            )

            version = "2023.9.3"
            readme_file_path = Path("README.md")
            readme_file_path.write_text(
                TEMPLATE_UPGRADE_VERSION_MARKDOWN.format(version),
                encoding="utf-8",
            )

            new_version = "2023.9.4"
            JavaVersionCommand(SemanticVersioningScheme).update_version(
                SemanticVersioningScheme.parse_version(new_version)
            )

            version_file_path.unlink()
            readme_file_path.unlink()


class ProjectFileJavaVersionCommandTestCase(unittest.TestCase):
    def test_project_file_not_found(self):
        with temp_directory(change_into=True):
            cmd = JavaVersionCommand(SemanticVersioningScheme)

            self.assertFalse(cmd.project_found())

    def test_project_file_found(self):
        with temp_file(name="upgradeVersion.json", change_into=True):
            cmd = JavaVersionCommand(SemanticVersioningScheme)

            self.assertTrue(cmd.project_found())



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json
from argparse import Namespace
from enum import IntEnum, auto
from typing import Optional

from pontos.enum import StrEnum
from pontos.errors import PontosError
from pontos.git import Git
from pontos.github.actions import ActionIO
from pontos.release.command import Command
from pontos.release.helper import ReleaseType, get_next_release_version
from pontos.terminal import Terminal
from pontos.typing import SupportsStr
from pontos.version import Version, VersionError
from pontos.version.helper import get_last_release_version
from pontos.version.schemes import VersioningScheme


class ShowReleaseReturnValue(IntEnum):
    """
    Possible return values of ReleaseCommand
    """

    SUCCESS = 0
    NO_LAST_RELEASE_VERSION = auto()
    NO_RELEASE_VERSION = auto()


class OutputFormat(StrEnum):
    ENV = "env"
    JSON = "json"
    GITHUB_ACTION = "github-action"


class ShowReleaseCommand(Command):
    def __init__(self, *, terminal: Terminal, error_terminal: Terminal) -> None:
        super().__init__(terminal=terminal, error_terminal=error_terminal)
        self.git = Git()

    def run(  # type: ignore[override]
        self,
        *,
        output_format: OutputFormat = OutputFormat.ENV,
        versioning_scheme: VersioningScheme,
        release_type: ReleaseType,
        release_version: Optional[Version],
        release_series: Optional[str] = None,
        git_tag_prefix: Optional[str] = None,
    ) -> int:
        git_tag_prefix = git_tag_prefix or ""

        try:
            last_release_version = get_last_release_version(
                parse_version=versioning_scheme.parse_version,
                git_tag_prefix=git_tag_prefix,
                tag_name=(
                    f"{git_tag_prefix}{release_series}.*"
                    if release_series
                    else None
                ),
            )
        except PontosError as e:
            last_release_version = None
            self.print_warning(f"Could not determine last release version. {e}")

        if not last_release_version and not release_version:
            self.print_error("Unable to determine last release version.")
            return ShowReleaseReturnValue.NO_LAST_RELEASE_VERSION

        calculator = versioning_scheme.calculator()

        try:
            release_version = get_next_release_version(
                last_release_version=last_release_version,
                calculator=calculator,
                release_type=release_type,
                release_version=release_version,
            )
        except VersionError as e:
            self.print_error(f"Unable to determine release version. {e}")
            return ShowReleaseReturnValue.NO_RELEASE_VERSION

        if last_release_version:
            last_release_version_dict = {
                "last_release_version": str(last_release_version),
                "last_release_version_major": last_release_version.major,
                "last_release_version_minor": last_release_version.minor,
                "last_release_version_patch": last_release_version.patch,
            }
        else:
            last_release_version_dict = {
                "last_release_version": "",
                "last_release_version_major": "",
                "last_release_version_minor": "",
                "last_release_version_patch": "",
            }

        if output_format == OutputFormat.JSON:
            release_dict = {
                "release_version": str(release_version),
                "release_version_major": release_version.major,
                "release_version_minor": release_version.minor,
                "release_version_patch": release_version.patch,
            }
            release_dict.update(last_release_version_dict)
            self.terminal.print(json.dumps(release_dict, indent=2))
        elif output_format == OutputFormat.GITHUB_ACTION:
            with ActionIO.out() as output:
                output.write(
                    "last_release_version",
                    last_release_version_dict["last_release_version"],
                )
                output.write(
                    "last_release_version_major",
                    last_release_version_dict["last_release_version_major"],
                )
                output.write(
                    "last_release_version_minor",
                    last_release_version_dict["last_release_version_minor"],
                )
                output.write(
                    "last_release_version_patch",
                    last_release_version_dict["last_release_version_patch"],
                )
                output.write("release_version_major", release_version.major)
                output.write("release_version_minor", release_version.minor)
                output.write("release_version_patch", release_version.patch)
                output.write("release_version", release_version)
        else:
            self.terminal.print(
                "LAST_RELEASE_VERSION="
                f"{last_release_version_dict['last_release_version']}"
            )
            self.terminal.print(
                "LAST_RELEASE_VERSION_MAJOR="
                f"{last_release_version_dict['last_release_version_major']}"
            )
            self.terminal.print(
                "LAST_RELEASE_VERSION_MINOR="
                f"{last_release_version_dict['last_release_version_minor']}"
            )
            self.terminal.print(
                "LAST_RELEASE_VERSION_PATCH="
                f"{last_release_version_dict['last_release_version_patch']}"
            )
            self.terminal.print(f"RELEASE_VERSION={release_version}")
            self.terminal.print(
                f"RELEASE_VERSION_MAJOR={release_version.major}"
            )
            self.terminal.print(
                f"RELEASE_VERSION_MINOR={release_version.minor}"
            )
            self.terminal.print(
                f"RELEASE_VERSION_PATCH={release_version.patch}"
            )

        return ShowReleaseReturnValue.SUCCESS


def show(
    args: Namespace,
    terminal: Terminal,
    error_terminal: Terminal,
    **_kwargs,
) -> SupportsStr:
    return ShowReleaseCommand(
        terminal=terminal,
        error_terminal=error_terminal,
    ).run(
        versioning_scheme=args.versioning_scheme,
        release_type=args.release_type,
        release_version=args.release_version,
        git_tag_prefix=args.git_tag_prefix,
        release_series=args.release_series,
        output_format=args.output_format,
    )



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .null import NullTerminal
from .rich import RichTerminal
from .terminal import Terminal

__all__ = (
    "Terminal",
    "NullTerminal",
    "RichTerminal",
)



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from contextlib import redirect_stderr
from io import StringIO

from pontos.version._parser import parse_args


class ParserTestCase(unittest.TestCase):
    def test_error_while_parsing(self):
        with redirect_stderr(StringIO()), self.assertRaises(SystemExit) as cm:
            parse_args(["update", "foo"])

        # exception code in argparse
        self.assertEqual(cm.exception.code, 2)



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pontos.terminal.null import NullTerminal
from pontos.terminal.rich import RichTerminal

from ._parser import parse_args


def main(args=None):
    parsed_args = parse_args(args)

    if parsed_args.quiet:
        term = NullTerminal()
    else:
        term = RichTerminal()

    parsed_args.func(term, parsed_args)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import base64
import gzip
from datetime import datetime
from typing import AsyncIterator, Iterable, Optional, Union

from pontos.github.api.client import GitHubAsyncREST
from pontos.github.api.helper import JSON_OBJECT
from pontos.github.models.base import SortOrder
from pontos.github.models.code_scanning import (
    AlertSort,
    AlertState,
    Analysis,
    CodeQLDatabase,
    CodeScanningAlert,
    DefaultSetup,
    DefaultSetupState,
    DismissedReason,
    Instance,
    Language,
    QuerySuite,
    SarifUploadInformation,
    Severity,
)
from pontos.helper import enum_or_value


class GitHubAsyncRESTCodeScanning(GitHubAsyncREST):
    async def _alerts(
        self,
        api: str,
        *,
        tool_name: Optional[str] = None,
        tool_guid: Optional[str] = "",
        severity: Union[Severity, str, None] = None,
        state: Union[AlertState, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[CodeScanningAlert]:
        params: dict[str, Union[str, None]] = {"per_page": "100"}

        if tool_name:
            params["tool_name"] = tool_name
        if tool_guid or tool_guid is None:
            params["tool_guid"] = tool_guid
        if severity:
            params["severity"] = enum_or_value(severity)
        if state:
            params["state"] = enum_or_value(state)
        if sort:
            params["sort"] = enum_or_value(sort)
        if direction:
            params["direction"] = enum_or_value(direction)

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for alert in response.json():
                yield CodeScanningAlert.from_dict(alert)

    async def organization_alerts(
        self,
        organization: str,
        *,
        tool_name: Optional[str] = None,
        tool_guid: Optional[str] = "",
        severity: Union[Severity, str, None] = None,
        state: Union[AlertState, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[CodeScanningAlert]:
        """
        Get the list of code scanning alerts for all repositories of a GitHub
        organization

        https://docs.github.com/en/rest/code-scanning/code-scanning#list-code-scanning-alerts-for-an-organization

        Args:
            organization: Name of the organization
            tool_name: The name of a code scanning tool. Only results by this
                tool will be listed. You can specify the tool by using either
                tool_name or tool_guid, but not both.
            tool_guid: The GUID of a code scanning tool. Only results by this
                tool will be listed. Note that some code scanning tools may not
                include a GUID in their analysis data. You can specify the tool
                by using either tool_guid or tool_name, but not both
            severity: If specified, only code scanning alerts with this severity
                will be returned
            state: Filter alerts by state
                resolutions
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the code scanning alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.code_scanning.organization_alerts(
                        "my-org"
                    ):
                        print(alert)
        """

        api = f"/orgs/{organization}/code-scanning/alerts"
        async for alert in self._alerts(
            api,
            state=state,
            severity=severity,
            tool_guid=tool_guid,
            tool_name=tool_name,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def alerts(
        self,
        repo: str,
        *,
        tool_name: Optional[str] = None,
        tool_guid: Optional[str] = "",
        severity: Union[Severity, str, None] = None,
        state: Union[AlertState, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[CodeScanningAlert]:
        """
        Get the list of code scanning alerts for a repository

        https://docs.github.com/en/rest/code-scanning/code-scanning#list-code-scanning-alerts-for-a-repository

        Args:
            repo: GitHub repository (owner/name)

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the code scanning alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.code_scanning.alerts(
                        "org/repo"
                    ):
                        print(alert)
        """

        api = f"/repos/{repo}/code-scanning/alerts"
        async for alert in self._alerts(
            api,
            state=state,
            severity=severity,
            tool_guid=tool_guid,
            tool_name=tool_name,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def alert(
        self,
        repo: str,
        alert_number: Union[str, int],
    ) -> CodeScanningAlert:
        """
        Get a single code scanning alert

        https://docs.github.com/en/rest/code-scanning/code-scanning#get-a-code-scanning-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a code scanning alert in
                its repository

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Code scanning alert information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    alert = await api.code_scanning.alert("foo/bar", 123)
        """
        api = f"/repos/{repo}/code-scanning/alerts/{alert_number}"
        response = await self._client.get(api)
        response.raise_for_status()
        return CodeScanningAlert.from_dict(response.json())

    async def update_alert(
        self,
        repo: str,
        alert_number: Union[str, int],
        state: Union[AlertState, str],
        *,
        dismissed_reason: Union[DismissedReason, str, None] = None,
        dismissed_comment: Optional[str] = None,
    ) -> CodeScanningAlert:
        """
        Update a single code scanning alert

        https://docs.github.com/en/rest/code-scanning/code-scanning#update-a-code-scanning-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a code scanning alert in
                its repository
            state: The state of the alert
            dismissed_reason: The reason for dismissing or closing the alert
            dismissed_comment: The dismissal comment associated with the
                dismissal of the alert.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Code scanning alert information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi
                from pontos.github.models.code_scanning import (
                    AlertState,
                    DismissedReason,
                )

                async with GitHubAsyncRESTApi(token) as api:
                    alert = await api.code_scanning.update_alert(
                        "foo/bar",
                        123,
                        AlertState.DISMISSED,
                        dismissed_reason=DismissedReason.WONT_FIX,
                        dismissed_comment="Not applicable",
                    )
        """
        api = f"/repos/{repo}/code-scanning/alerts/{alert_number}"

        data = {"state": enum_or_value(state)}
        if dismissed_reason:
            data["dismissed_reason"] = enum_or_value(dismissed_reason)
        if dismissed_comment:
            data["dismissed_comment"] = dismissed_comment

        response = await self._client.patch(api, data=data)
        response.raise_for_status()
        return CodeScanningAlert.from_dict(response.json())

    async def instances(
        self,
        repo: str,
        alert_number: Union[str, int],
        *,
        ref: Optional[str] = None,
    ) -> AsyncIterator[Instance]:
        """
        Lists all instances of the specified code scanning alert

        https://docs.github.com/en/rest/code-scanning/code-scanning#list-instances-of-a-code-scanning-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a code scanning alert in
                its repository
            ref: The Git reference for the results you want to list. The ref
                for a branch can be formatted either as refs/heads/<branch name>
                or simply <branch name>. To reference a pull request use
                refs/pull/<number>/merge.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the code scanning alert instances

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for instance in api.code_scanning.instances(
                        "org/repo", 1
                    ):
                        print(instance)
        """

        api = f"/repos/{repo}/code-scanning/alerts/{alert_number}/instances"
        params = {"per_page": "100"}

        if ref:
            params["ref"] = ref

        async for response in self._client.get_all(api, params=params):
            for alert in response.json():
                yield Instance.from_dict(alert)

    async def analyses(
        self,
        repo: str,
        *,
        tool_name: Optional[str] = None,
        tool_guid: Optional[str] = "",
        ref: Optional[str] = None,
        sarif_id: Optional[str] = None,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[Analysis]:
        """
        Lists the details of all code scanning analyses for a repository,
        starting with the most recent.

        https://docs.github.com/en/rest/code-scanning/code-scanning#list-code-scanning-analyses-for-a-repository

        Args:
            repo: GitHub repository (owner/name)
            tool_name: The name of a code scanning tool. Only results by this
                tool will be listed. You can specify the tool by using either
                tool_name or tool_guid, but not both.
            tool_guid: The GUID of a code scanning tool. Only results by this
                tool will be listed. Note that some code scanning tools may not
                include a GUID in their analysis data. You can specify the tool
                by using either tool_guid or tool_name, but not both
            ref: The Git reference for the analyses you want to list. The ref
                for a branch can be formatted either as refs/heads/<branch name>
                or simply <branch name>. To reference a pull request use
                refs/pull/<number>/merge.
            sarif_id: Filter analyses belonging to the same SARIF upload

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the code scanning alert analysis data

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for data in api.code_scanning.analyses(
                        "org/repo"
                    ):
                        print(data)
        """

        api = f"/repos/{repo}/code-scanning/analyses"
        params: dict[str, Union[str, None]] = {"per_page": "100"}

        if tool_name:
            params["tool_name"] = tool_name
        if tool_guid or tool_guid is None:
            params["tool_guid"] = tool_guid
        if ref:
            params["ref"] = ref
        if sarif_id:
            params["sarif_id"] = sarif_id
        if direction:
            params["direction"] = enum_or_value(direction)

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for alert in response.json():
                yield Analysis.from_dict(alert)

    async def analysis(
        self,
        repo: str,
        analysis_id: Union[int, str],
    ) -> Analysis:
        """
        Gets a specified code scanning analysis for a repository

        https://docs.github.com/en/rest/code-scanning/code-scanning#get-a-code-scanning-analysis-for-a-repository

        Args:
            repo: GitHub repository (owner/name)
            analysis_id: The ID of the analysis

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Code scanning alert analysis data

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    analysis = await api.code_scanning.analysis(
                        "org/repo", 123
                    )
                    print(analysis)
        """

        api = f"/repos/{repo}/code-scanning/analyses/{analysis_id}"
        response = await self._client.get(api)
        response.raise_for_status()
        return Analysis.from_dict(response.json())

    async def delete_analysis(
        self,
        repo: str,
        analysis_id: Union[int, str],
    ) -> dict[str, str]:
        """
        Delete a specified code scanning analysis from a repository

        https://docs.github.com/en/rest/code-scanning/code-scanning#delete-a-code-scanning-analysis-from-a-repository

        Args:
            repo: GitHub repository (owner/name)
            analysis_id: The ID of the analysis

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            See the GitHub documentation for the response object

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    await api.code_scanning.delete(
                        "org/repo", 123
                    )
        """

        api = f"/repos/{repo}/code-scanning/analyses/{analysis_id}"
        response = await self._client.delete(api)
        response.raise_for_status()
        return response.json()

    async def codeql_databases(
        self,
        repo: str,
    ) -> AsyncIterator[CodeQLDatabase]:
        """
        List the CodeQL databases that are available in a repository.

        https://docs.github.com/en/rest/code-scanning/code-scanning#list-codeql-databases-for-a-repository

        Args:
            repo: GitHub repository (owner/name)

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the code scanning codeql database
            information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for database in api.code_scanning.codeql_databases(
                        "org/repo"
                    ):
                        print(database)
        """

        api = f"/repos/{repo}/code-scanning/codeql/databases"
        params = {"per_page": "100"}

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for alert in response.json():
                yield CodeQLDatabase.from_dict(alert)

    async def codeql_database(
        self,
        repo: str,
        language: str,
    ) -> CodeQLDatabase:
        """
        Get a CodeQL database for a language in a repository

        https://docs.github.com/en/rest/code-scanning/code-scanning#get-a-codeql-database-for-a-repository

        Args:
            repo: GitHub repository (owner/name)
            language: The language of the CodeQL database

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Code scanning CodeQL database information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    db = await api.code_scanning.codeql_database(
                        "org/repo", "java"
                    )
                    print(db)
        """

        api = f"/repos/{repo}/code-scanning/codeql/databases/{language}"
        response = await self._client.get(api)
        response.raise_for_status()
        return CodeQLDatabase.from_dict(response.json())

    async def default_setup(
        self,
        repo: str,
    ) -> DefaultSetup:
        """
        Gets a code scanning default setup configuration

        https://docs.github.com/en/rest/code-scanning/code-scanning#get-a-code-scanning-default-setup-configuration

        Args:
            repo: GitHub repository (owner/name)

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Code scanning default setup

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    setup = await api.code_scanning.default_setup(
                        "org/repo"
                    )
                    print(setup)
        """

        api = f"/repos/{repo}/code-scanning/default-setup"
        response = await self._client.get(api)
        response.raise_for_status()
        return DefaultSetup.from_dict(response.json())

    async def update_default_setup(
        self,
        repo: str,
        state: Union[str, DefaultSetupState],
        query_suite: Union[str, QuerySuite],
        languages: Iterable[Union[str, Language]],
    ) -> dict[str, str]:
        """
        Updates a code scanning default setup configuration

        https://docs.github.com/en/rest/code-scanning/code-scanning#update-a-code-scanning-default-setup-configuration

        Args:
            repo: GitHub repository (owner/name)
            state: Whether code scanning default setup has been configured or
                not
            query_suite: CodeQL query suite to be used
            languages: CodeQL languages to be analyzed

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            See the GitHub documentation for the response object

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi
                from pontos.github.models.code_scanning import (
                    DefaultSetupState,
                    Language,
                    QuerySuite,
                )

                async with GitHubAsyncRESTApi(token) as api:
                    await api.code_scanning.update_default_setup(
                        "org/repo",
                        state=DefaultSetupState.CONFIGURED,
                        query_suite=QuerySuite.EXTENDED,
                        languages=[Language.PYTHON, Language.JAVASCRIPT]
                    )
        """

        api = f"/repos/{repo}/code-scanning/code-scanning/default-setup"
        data = {
            "state": enum_or_value(state),
            "query_suite": enum_or_value(query_suite),
            "languages": [enum_or_value(value) for value in languages],
        }
        response = await self._client.patch(api, data=data)
        response.raise_for_status()
        return response.json()

    async def upload_sarif_data(
        self,
        repo: str,
        commit_sha: str,
        ref: str,
        sarif: bytes,
        *,
        checkout_uri: Optional[str] = None,
        started_at: Optional[datetime] = None,
        tool_name: Optional[str] = None,
        validate: Optional[bool] = None,
    ) -> dict[str, str]:
        """
        Upload SARIF data containing the results of a code scanning analysis to
        make the results available in a repository

        https://docs.github.com/en/rest/code-scanning/code-scanning#upload-an-analysis-as-sarif-data

        Args:
            repo: GitHub repository (owner/name)
            commit_sha: The SHA of the commit to which the analysis you are
                uploading relates
            ref: The full Git reference, formatted as refs/heads/<branch name>,
                refs/pull/<number>/merge, or refs/pull/<number>/head
            sarif:
            checkout_uri: The base directory used in the analysis, as it appears
                in the SARIF file
            started_at: The time that the analysis run began
            tool_name: The name of the tool used to generate the code scanning
                analysis
            validate: Whether the SARIF file will be validated according to the
                code scanning specifications

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            See the GitHub documentation for the response object

        Example:
            .. code-block:: python

                from pathlib import Path
                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    json = await api.code_scanning.upload_sarif_data(
                        "org/repo",
                        commit_sha="4b6472266afd7b471e86085a6659e8c7f2b119da",
                        ref="refs/heads/main",
                        sarif=Path("/path/to/sarif.file").read_bytes(),
                    )
                    print(json["id"])
        """
        api = f"/repos/{repo}/code-scanning/sarifs"
        data: JSON_OBJECT = {
            "commit_sha": commit_sha,
            "ref": ref,
        }
        if checkout_uri:
            data["checkout_uri"] = checkout_uri
        if started_at:
            data["started_at"] = started_at.isoformat(timespec="seconds")
        if tool_name:
            data["tool_name"] = tool_name
        if validate is not None:
            data["validate"] = validate

        compressed = gzip.compress(sarif, mtime=0)
        encoded = base64.b64encode(compressed).decode(encoding="ascii")

        data["sarif"] = encoded

        response = await self._client.post(api, data=data)
        response.raise_for_status()
        return response.json()

    async def sarif(self, repo: str, sarif_id: str) -> SarifUploadInformation:
        """
        Gets information about a SARIF upload, including the status and the URL
        of the analysis that was uploaded so that you can retrieve details of
        the analysis

        https://docs.github.com/en/rest/code-scanning/code-scanning#get-information-about-a-sarif-upload

        Args:
            repo: GitHub repository (owner/name)
            sarif_id: The SARIF ID obtained after uploading

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Information about the SARIF upload

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    sarif = await api.code_scanning.sarif(
                        "org/repo",
                        "47177e22-5596-11eb-80a1-c1e54ef945c6",
                    )
                    print(sarif)
        """
        api = f"/repos/{repo}/code-scanning/sarifs/{sarif_id}"

        response = await self._client.get(api)
        response.raise_for_status()
        return SarifUploadInformation.from_dict(response.json())



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys
from typing import NoReturn, Optional, Sequence

from pontos.changelog.conventional_commits import ChangelogBuilder
from pontos.errors import PontosError
from pontos.terminal.null import NullTerminal
from pontos.terminal.rich import RichTerminal
from pontos.version.helper import get_last_release_version

from ._parser import parse_args


def main(args: Optional[Sequence[str]] = None) -> NoReturn:
    parsed_args = parse_args(args)

    term = NullTerminal() if parsed_args.quiet else RichTerminal()

    if parsed_args.current_version:
        last_version = parsed_args.current_version
    else:
        last_version = get_last_release_version(
            parsed_args.versioning_scheme.parse_version,
            git_tag_prefix=parsed_args.git_tag_prefix,
        )

    try:
        changelog_builder = ChangelogBuilder(
            config=parsed_args.config,
            repository=parsed_args.repository,
        )
        if parsed_args.output:
            changelog_builder.create_changelog_file(
                parsed_args.output,
                last_version=last_version,
                next_version=parsed_args.next_version,
            )
        else:
            changelog = changelog_builder.create_changelog(
                last_version=last_version,
                next_version=parsed_args.next_version,
            )
            term.out(changelog)
    except KeyboardInterrupt:
        sys.exit(1)
    except PontosError as e:
        term.error(str(e))
        sys.exit(2)

    sys.exit(0)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import dataclass
from datetime import datetime
from typing import Optional
from uuid import UUID

from pontos.models import Model, StrEnum


class EventName(StrEnum):
    NEW_CVE_RECEIVED = "New CVE Received"
    INITIAL_ANALYSIS = "Initial Analysis"
    REANALYSIS = "Reanalysis"
    CVE_MODIFIED = "CVE Modified"
    MODIFIED_ANALYSIS = "Modified Analysis"
    CVE_TRANSLATED = "CVE Translated"
    VENDOR_COMMENT = "Vendor Comment"
    CVE_SOURCE_UPDATE = "CVE Source Update"
    CPE_DEPRECATION_REMAP = "CPE Deprecation Remap"
    CWE_REMAP = "CWE Remap"
    CVE_REJECTED = "CVE Rejected"
    CVE_UNREJECTED = "CVE Unrejected"
    CVE_CISA_KEV_UPDATE = "CVE CISA KEV Update"
    REFERENCE_TAG_UPDATE = "Reference Tag Update"


@dataclass
class Detail:
    type: str
    action: Optional[str] = None
    old_value: Optional[str] = None
    new_value: Optional[str] = None


@dataclass
class CVEChange(Model):
    cve_id: str
    event_name: EventName
    cve_change_id: UUID
    source_identifier: str
    created: Optional[datetime] = None
    details: Optional[list[Detail]] = None



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from datetime import datetime
from typing import Any, AsyncIterator, Dict, Optional, Union

from pontos.github.api.client import GitHubAsyncREST
from pontos.github.models.tag import GitObjectType, RepositoryTag, Tag
from pontos.helper import enum_or_value


class GitHubAsyncRESTTags(GitHubAsyncREST):
    async def create(
        self,
        repo: str,
        tag: str,
        message: str,
        name: str,
        email: str,
        git_object: str,
        *,
        git_object_type: Optional[
            Union[GitObjectType, str]
        ] = GitObjectType.COMMIT,
        date: Optional[datetime] = None,
    ) -> Tag:
        """
        Create a new Git tag

        https://docs.github.com/en/rest/git/tags#create-a-tag-object

        Args:
            repo: GitHub repository (owner/name) to use
            tag: The tag's name. This is typically a version (e.g., "v0.0.1").
            message: The tag message.
            name: The name of the author of the tag
            email: The email of the author of the tag
            git_object: The SHA of the git object this is tagging.
            git_object_type: The type of the object we're tagging. Normally this
                is a commit type but it can also be a tree or a blob.
            date: When this object was tagged.

        Raises:
            HTTPStatusError: If the request was invalid

        Returns:
            A new git tag

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    tag = await api.tags.create(
                        "foo/bar",
                        "v1.2.3",
                        "Create tag v1.2.3",
                        "John Doe",
                        "john@doe.com",
                        e746420,
                    )
                    print(tag)
        """
        data = {
            "tag": tag,
            "message": message,
            "object": git_object,
            "type": enum_or_value(git_object_type),
            "tagger": {
                "name": name,
                "email": email,
            },
        }

        if date:
            data["tagger"]["date"] = date.isoformat(timespec="seconds")

        api = f"/repos/{repo}/git/tags"
        response = await self._client.post(api, data=data)
        response.raise_for_status()
        return Tag.from_dict(response.json())

    async def create_tag_reference(
        self,
        repo: str,
        tag: str,
        sha: str,
    ) -> None:
        """
        Create git tag reference (A real tag in git).

        https://docs.github.com/en/rest/git/refs#create-a-reference

        Args:
            repo: The name of the repository.
                The name is not case sensitive.
            tag: Github tag name.
            sha: The SHA1 value for this Github tag.

        Raises:
            HTTPStatusError: If the request was invalid

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    await api.tags.create_tag_reference(
                        "foo/bar",
                        "v1.2.3",
                        e746420,
                    )
        """

        data: Dict[str, Any] = {
            "ref": f"refs/tags/{tag}",
            "sha": sha,
        }

        api = f"/repos/{repo}/git/refs"
        response = await self._client.post(api, data=data)
        response.raise_for_status()

    async def get(self, repo: str, tag_sha: str) -> Tag:
        """
        Get information about a git tag

        Args:
            repo: GitHub repository (owner/name) to use
            tag_sha: SHA of the git tag object

        Raises:
            HTTPStatusError: If the request was invalid

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    tag = await api.tags.get("foo/bar", "e746420")
                    print(tag)
        """
        api = f"/repos/{repo}/git/tags/{tag_sha}"
        response = await self._client.get(api)
        response.raise_for_status()
        return Tag.from_dict(response.json())

    async def get_all(self, repo: str) -> AsyncIterator[RepositoryTag]:
        """
        Get information about all git tags

        Args:
            repo: GitHub repository (owner/name) to use

        Raises:
            HTTPStatusError: If the request was invalid

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for tag in api.tags.get_all(
                        "foo/bar"
                    ):
                        print(tag)
        """
        api = f"/repos/{repo}/git/tags"
        params = {"per_page": "100"}

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()
            for tag in response.json():
                yield RepositoryTag.from_dict(tag)



"""
This script delete a package from a repository, if it contains the specified tag.
"""

from argparse import ArgumentParser, Namespace

from pontos.github.api import GitHubAsyncRESTApi
from pontos.github.models.packages import PackageType


def package_type(value: str) -> PackageType:
    if isinstance(value, PackageType):
        return value
    return PackageType(value.lower())


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("organization", help="organization name")
    parser.add_argument("package", help="package name")
    parser.add_argument(
        "--package-type",
        type=package_type,
        help="package type",
        default=PackageType.CONTAINER,
    )
    parser.add_argument("tag", help="The tag to be deleted.")


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    if not await api.packages.exists(
        organization=args.organization,
        package_name=args.package,
        package_type=args.package_type,
    ):
        print(
            f"Package {args.package} does not exist in organization {args.organization}"
        )
        return 1
    print(f"Found package {args.package} in organization {args.organization}")

    await api.packages.delete_package_with_tag(
        organization=args.organization,
        package_name=args.package,
        package_type=args.package_type,
        tag=args.tag,
    )
    print(
        f"Deleted tag {args.tag} from package {args.package} in organization {args.organization}"
    )
    return 0



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=invalid-name

from pontos import updateheader

if __name__ == "__main__":
    updateheader.main()



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from contextlib import contextmanager
from typing import Any, Generator

from pontos.helper import DownloadProgressIterable

from .terminal import Terminal


class NullTerminal(Terminal):
    """A terminal implementation to keep the terminal quiet"""

    @contextmanager
    def indent(self, indentation: int = 4) -> Generator[None, None, None]:
        yield

    def out(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def print(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def ok(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def fail(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def error(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def warning(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def info(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def bold_info(self, *messages: Any, **kwargs: Any) -> None:
        pass

    def download_progress(self, progress: DownloadProgressIterable) -> None:
        progress.run()



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=line-too-long, arguments-differ, redefined-builtin
# ruff: noqa: E501

from datetime import datetime, timezone
from itertools import repeat
from typing import Any, Optional
from unittest.mock import MagicMock, patch
from uuid import UUID, uuid4

from httpx import AsyncClient, Response

from pontos.errors import PontosError
from pontos.nvd.api import now
from pontos.nvd.cpe.api import MAX_CPES_PER_PAGE, CPEApi
from tests import AsyncMock, IsolatedAsyncioTestCase, aiter, anext
from tests.nvd import get_cpe_data


def uuid_replace_str(uuid: UUID, iteration: int, number: int) -> str:
    id_str = str(uuid).rsplit("-", 2)
    nn = "".join([str(j) for j in repeat(number, 4)])
    ii = "".join([str(j) for j in repeat(iteration, 12)])
    return f"{id_str[0]}-{nn}-{ii}"


def uuid_replace(uuid: UUID, iteration: int, number: int) -> UUID:
    return UUID(uuid_replace_str(uuid, iteration, number))


def create_cpe_response(
    cpe_name_id: UUID,
    *,
    update: Optional[dict[str, Any]] = None,
    results: int = 1,
    iteration: int = 1,
) -> MagicMock:
    products = [
        {
            "cpe": get_cpe_data(
                {
                    "cpe_name_id": f"{uuid_replace_str(cpe_name_id, iteration, i)}"
                }
            )
        }
        for i in range(1, results + 1)
    ]

    data = {
        "products": products,
        "results_per_page": results,
    }
    if update:
        data.update(update)

    response = MagicMock(spec=Response)
    response.json.return_value = data
    return response


def create_cpes_responses(
    cpe_name_id: UUID, responses: int = 2, results_per_response: int = 1
) -> list[MagicMock]:
    return [
        create_cpe_response(
            cpe_name_id=cpe_name_id,
            update={"total_results": responses * results_per_response},
            results=results_per_response,
            iteration=i,
        )
        for i in range(1, responses + 1)
    ]


class CPEApiTestCase(IsolatedAsyncioTestCase):
    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    def setUp(self, async_client: MagicMock, monotonic_mock: MagicMock) -> None:
        self.http_client = AsyncMock()
        async_client.return_value = self.http_client
        monotonic_mock.return_value = 0
        self.api = CPEApi()

    async def test_no_cpe_name_id(self):
        with self.assertRaises(PontosError):
            await self.api.cpe(None)

    async def test_no_cpe(self):
        data = {
            "products": [],
            "results_per_page": 1,
        }
        response = MagicMock(spec=Response)
        response.json.return_value = data
        self.http_client.get.return_value = response

        with self.assertRaises(PontosError):
            await self.api.cpe("CPE-1")

    async def test_cpe(self):
        uuid = uuid4()
        self.http_client.get.return_value = create_cpe_response(uuid)

        cpe = await self.api.cpe(uuid)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={"cpeNameId": str(uuid)},
        )

        self.assertEqual(
            cpe.cpe_name,
            "cpe:2.3:o:microsoft:windows_10_22h2:-:*:*:*:*:*:arm64:*",
        )
        self.assertEqual(cpe.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.assertFalse(cpe.deprecated)
        self.assertEqual(
            cpe.last_modified,
            datetime(2022, 12, 9, 18, 15, 16, 973000, tzinfo=timezone.utc),
        )
        self.assertEqual(
            cpe.created,
            datetime(2022, 12, 9, 16, 20, 6, 943000, tzinfo=timezone.utc),
        )

        self.assertEqual(cpe.refs, [])
        self.assertEqual(cpe.titles, [])
        self.assertEqual(cpe.deprecated_by, [])

    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    async def test_rate_limit(
        self,
        sleep_mock: MagicMock,
        monotonic_mock: MagicMock,
    ):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(uuid, 8)
        monotonic_mock.side_effect = [10, 11]

        it = aiter(self.api.cpes())
        await anext(it)
        await anext(it)
        await anext(it)
        await anext(it)
        await anext(it)

        sleep_mock.assert_not_called()

        await anext(it)

        sleep_mock.assert_called_once_with(20.0)

    @patch("pontos.nvd.cpe.api.now", spec=now)
    async def test_cves_last_modified_start_date(self, now_mock: MagicMock):
        uuid = uuid4()
        now_mock.return_value = datetime(2022, 12, 31)
        self.http_client.get.side_effect = create_cpes_responses(uuid)

        it = aiter(
            self.api.cpes(last_modified_start_date=datetime(2022, 12, 1))
        )
        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CPES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 1,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_last_modified_end_date(self):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(uuid)

        it = aiter(
            self.api.cpes(
                last_modified_start_date=datetime(2022, 12, 1),
                last_modified_end_date=datetime(2022, 12, 31),
            )
        )
        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CPES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 1,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cpes_keywords(self):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(uuid)

        it = aiter(self.api.cpes(keywords=["Mac OS X", "kernel"]))
        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "keywordSearch": "Mac OS X kernel",
                "keywordExactMatch": "",
                "resultsPerPage": MAX_CPES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "keywordSearch": "Mac OS X kernel",
                "keywordExactMatch": "",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cpes_keyword(self):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(uuid)

        it = aiter(self.api.cpes(keywords="macOS"))
        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "keywordSearch": "macOS",
                "resultsPerPage": MAX_CPES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "keywordSearch": "macOS",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cpes_cpe_match_string(self):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(uuid)

        it = aiter(
            self.api.cpes(
                cpe_match_string="cpe:2.3:o:microsoft:windows_10:20h2:*:*:*:*:*:*:*"
            )
        )
        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "resultsPerPage": MAX_CPES_PER_PAGE,
                "cpeMatchString": "cpe:2.3:o:microsoft:windows_10:20h2:*:*:*:*:*:*:*",
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "cpeMatchString": "cpe:2.3:o:microsoft:windows_10:20h2:*:*:*:*:*:*:*",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cpes_match_criteria_id(self):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(uuid)

        it = aiter(
            self.api.cpes(
                match_criteria_id="36FBCF0F-8CEE-474C-8A04-5075AF53FAF4"
            )
        )
        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "matchCriteriaId": "36FBCF0F-8CEE-474C-8A04-5075AF53FAF4",
                "resultsPerPage": MAX_CPES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "matchCriteriaId": "36FBCF0F-8CEE-474C-8A04-5075AF53FAF4",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cpes_request_results(self):
        uuid = uuid4()
        self.http_client.get.side_effect = create_cpes_responses(
            uuid, results_per_response=2
        )

        it = aiter(self.api.cpes(request_results=10))
        cve = await anext(it)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 0,
                "resultsPerPage": 10,
            },
        )
        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 1))

        self.http_client.get.reset_mock()
        cve = await anext(it)
        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 1, 2))
        self.http_client.get.assert_not_called()

        self.http_client.get.reset_mock()
        cve = await anext(it)
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpes/2.0",
            headers={},
            params={
                "startIndex": 2,
                "resultsPerPage": 2,
            },
        )
        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 1))

        self.http_client.get.reset_mock()
        cve = await anext(it)
        self.assertEqual(cve.cpe_name_id, uuid_replace(uuid, 2, 2))
        self.http_client.get.assert_not_called()

        self.http_client.get.reset_mock()

        with self.assertRaises(Exception):
            cve = await anext(it)

    async def test_context_manager(self):
        async with self.api:
            pass

        self.http_client.__aenter__.assert_awaited_once()
        self.http_client.__aexit__.assert_awaited_once()



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script prints out all artifacts of a given repository
"""

from argparse import ArgumentParser, Namespace

from rich.console import Console
from rich.table import Table

from pontos.github.api import GitHubAsyncRESTApi


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("repository")


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    table = Table()
    table.add_column("Name")
    table.add_column("ID")
    table.add_column("URL")
    table.add_column("Updated")
    table.add_column("Expired")
    table.add_column("Size (in KB)", justify="right")

    count = 0
    async for artifact in api.artifacts.get_all(args.repository):
        table.add_row(
            artifact.name,
            str(artifact.id),
            f"[link={artifact.archive_download_url}]"
            f"{artifact.archive_download_url}[/link]",
            str(artifact.updated_at),
            str(artifact.expired),
            f"{artifact.size_in_bytes / 1024:.2f}",
        )
        count += 1

    console = Console()
    console.print(table)

    print(f"{count} artifacts.")
    return 0



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable = protected-access

import unittest
from pathlib import Path
from unittest.mock import MagicMock, PropertyMock, patch

import tomlkit

from pontos.testing import temp_directory, temp_file, temp_python_module
from pontos.version import VersionError
from pontos.version.commands._python import PythonVersionCommand
from pontos.version.schemes import (
    PEP440VersioningScheme,
    SemanticVersioningScheme,
)


class GetCurrentPythonVersionCommandTestCase(unittest.TestCase):
    def test_missing_tool_pontos_version_section(self):
        with (
            temp_file("[tool.pontos]", name="pyproject.toml", change_into=True),
            self.assertRaisesRegex(
                VersionError,
                r"^\[tool\.pontos\.version\] section missing in .*\.$",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

    def test_missing_version_module_file_key(self):
        with (
            temp_file(
                '[tool.pontos.version]\nname="foo"',
                name="pyproject.toml",
                change_into=True,
            ),
            self.assertRaisesRegex(
                VersionError,
                r"^version-module-file key not set in \[tool\.pontos\.version\] "
                r"section .*\.$",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

    def test_version_file_path(self):
        with temp_file(
            '[tool.pontos.version]\nversion-module-file="foo/__version__.py"',
            name="pyproject.toml",
            change_into=True,
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)

            self.assertEqual(
                cmd.version_file_path, Path("foo") / "__version__.py"
            )

    def test_pyproject_toml_file_not_exists(self):
        with (
            temp_directory(change_into=True),
            self.assertRaisesRegex(
                VersionError, "pyproject.toml file not found."
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

    def test_no_version_module(self):
        with (
            temp_file(
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                name="pyproject.toml",
                change_into=True,
            ),
            self.assertRaisesRegex(
                VersionError,
                r"Could not load version from 'foo'\. .* not found.",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            cmd.get_current_version()

    def test_get_current_version(self):
        with temp_python_module(
            "__version__ = '1.2.3'", name="foo", change_into=True
        ) as tmp_module:
            tmp_file = tmp_module.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            version = cmd.get_current_version()

            self.assertEqual(
                version, PEP440VersioningScheme.parse_version("1.2.3")
            )

    def test_get_current_semantic_version(self):
        with temp_python_module(
            "__version__ = '1.2.3a1'", name="foo", change_into=True
        ) as tmp_module:
            tmp_file = tmp_module.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3a1"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(SemanticVersioningScheme)
            version = cmd.get_current_version()

            self.assertEqual(
                version, PEP440VersioningScheme.parse_version("1.2.3a1")
            )
            self.assertIsInstance(version, PEP440VersioningScheme.version_cls)


class UpdatePythonVersionTestCase(unittest.TestCase):
    def test_update_version_file(self):
        content = "__version__ = '21.1'"
        with temp_python_module(content, name="foo", change_into=True) as temp:
            tmp_file = temp.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )

            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.2")
            previous_version = PEP440VersioningScheme.parse_version("21.1")

            updated = cmd.update_version(new_version)

            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(
                updated.changed_files, [Path("foo.py"), tmp_file.resolve()]
            )

            text = temp.read_text(encoding="utf8")

        *_, version_line, _last_line = text.split("\n")

        self.assertEqual(version_line, '__version__ = "22.2"')

    def test_empty_pyproject_toml(self):
        with (
            temp_file("", name="pyproject.toml", change_into=True),
            self.assertRaisesRegex(
                VersionError,
                r"\[tool.pontos.version\] section missing in .*pyproject\.toml\.",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.1.2")
            cmd.update_version(new_version)

    def test_empty_tool_section(self):
        with (
            temp_file("[tool]", name="pyproject.toml", change_into=True),
            self.assertRaisesRegex(
                VersionError,
                r"\[tool.pontos.version\] section missing in .*pyproject\.toml\.",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.1.2")
            cmd.update_version(new_version)

    def test_empty_tool_poetry_section(self):
        content = "__version__ = '22.1'"
        with temp_python_module(content, name="foo", change_into=True) as temp:
            tmp_file = temp.parent / "pyproject.toml"
            tmp_file.write_text(
                "[tool.poetry]\n"
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.2")
            previous_version = PEP440VersioningScheme.parse_version("22.1")
            updated = cmd.update_version(new_version)

            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(
                updated.changed_files, [Path("foo.py"), tmp_file.resolve()]
            )

            text = tmp_file.read_text(encoding="utf8")

            toml = tomlkit.parse(text)

            self.assertEqual(toml["tool"]["poetry"]["version"], "22.2")

    def test_override_existing_version(self):
        content = "__version__ = '1.2.3'"
        with temp_python_module(content, name="foo", change_into=True) as temp:
            tmp_file = temp.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.2")
            previous_version = PEP440VersioningScheme.parse_version("1.2.3")
            updated = cmd.update_version(new_version)

            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(
                updated.changed_files, [Path("foo.py"), tmp_file.resolve()]
            )

            text = tmp_file.read_text(encoding="utf8")

            toml = tomlkit.parse(text)

            self.assertEqual(toml["tool"]["poetry"]["version"], "22.2")

    def test_development_version(self):
        content = "__version__ = '1.2.3'"
        with temp_python_module(content, name="foo", change_into=True) as temp:
            tmp_file = temp.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.2.dev1")
            previous_version = PEP440VersioningScheme.parse_version("1.2.3")
            updated = cmd.update_version(new_version)

            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(
                updated.changed_files, [Path("foo.py"), tmp_file.resolve()]
            )

            text = tmp_file.read_text(encoding="utf8")

            toml = tomlkit.parse(text)

            self.assertEqual(toml["tool"]["poetry"]["version"], "22.2.dev1")

    def test_no_update(self):
        content = "__version__ = '1.2.3'"
        with temp_python_module(content, name="foo", change_into=True) as temp:
            tmp_file = temp.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("1.2.3")
            updated = cmd.update_version(new_version)

            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.previous, new_version)
            self.assertEqual(updated.changed_files, [])

    def test_forced_updated(self):
        content = "__version__ = '1.2.3'"
        with temp_python_module(content, name="foo", change_into=True) as temp:
            tmp_file = temp.parent / "pyproject.toml"
            tmp_file.write_text(
                '[tool.poetry]\nversion = "1.2.3"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("1.2.3")
            updated = cmd.update_version(new_version, force=True)

            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.previous, new_version)
            self.assertEqual(
                updated.changed_files, [Path("foo.py"), tmp_file.resolve()]
            )

            text = tmp_file.read_text(encoding="utf8")

            toml = tomlkit.parse(text)

            self.assertEqual(toml["tool"]["poetry"]["version"], "1.2.3")


class VerifyVersionTestCase(unittest.TestCase):
    def test_current_version_not_equal_pyproject_toml_version(self):
        fake_version_py = Path("foo.py")
        with (
            patch.object(
                PythonVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=PEP440VersioningScheme.parse_version("1.2.3")
                ),
            ),
            patch.object(
                PythonVersionCommand,
                "version_file_path",
                new=PropertyMock(return_value=fake_version_py),
            ),
            self.assertRaisesRegex(
                VersionError,
                "The version .* in .* doesn't match the current version .*.",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            version = PEP440VersioningScheme.parse_version("1.2.3")
            cmd.verify_version(version)

    def test_current_version(self):
        fake_version_py = Path("foo.py")
        content = (
            '[tool.poetry]\nversion = "1.2.3"\n'
            '[tool.pontos.version]\nversion-module-file = "foo.py"'
        )

        with (
            temp_file(content, name="pyproject.toml", change_into=True),
            patch.object(
                PythonVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=PEP440VersioningScheme.parse_version("1.2.3")
                ),
            ),
            patch.object(
                PythonVersionCommand,
                "version_file_path",
                new=PropertyMock(return_value=fake_version_py),
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            cmd.verify_version("current")

    def test_current_failure(self):
        fake_version_py = Path("foo.py")
        content = (
            '[tool.poetry]\nversion = "1.2.4"\n'
            '[tool.pontos.version]\nversion-module-file = "foo.py"'
        )

        with (
            temp_file(content, name="pyproject.toml", change_into=True),
            patch.object(
                PythonVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=PEP440VersioningScheme.parse_version("1.2.3")
                ),
            ),
            patch.object(
                PythonVersionCommand,
                "version_file_path",
                new=PropertyMock(return_value=fake_version_py),
            ),
            self.assertRaisesRegex(
                VersionError,
                "The version .* in .* doesn't match the current version .*.",
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            cmd.verify_version("current")

    def test_provided_version_mismatch(self):
        fake_version_py = Path("foo.py")
        content = (
            '[tool.poetry]\nversion = "1.2.3"\n'
            '[tool.pontos.version]\nversion-module-file = "foo.py"'
        )

        with (
            temp_file(content, name="pyproject.toml", change_into=True),
            patch.object(
                PythonVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=PEP440VersioningScheme.parse_version("1.2.3")
                ),
            ),
            patch.object(
                PythonVersionCommand,
                "version_file_path",
                new=PropertyMock(return_value=fake_version_py),
            ),
        ):
            with self.assertRaisesRegex(
                VersionError,
                "Provided version .* does not match the current version .*.",
            ):
                cmd = PythonVersionCommand(PEP440VersioningScheme)
                version = PEP440VersioningScheme.parse_version("1.2.4")
                cmd.verify_version(version)

    def test_verify_success(self):
        fake_version_py = Path("foo.py")
        content = (
            '[tool.poetry]\nversion = "1.2.3"\n'
            '[tool.pontos.version]\nversion-module-file = "foo.py"'
        )

        with (
            temp_file(content, name="pyproject.toml", change_into=True),
            patch.object(
                PythonVersionCommand,
                "get_current_version",
                MagicMock(
                    return_value=PEP440VersioningScheme.parse_version("1.2.3")
                ),
            ),
            patch.object(
                PythonVersionCommand,
                "version_file_path",
                new=PropertyMock(return_value=fake_version_py),
            ),
        ):
            cmd = PythonVersionCommand(PEP440VersioningScheme)
            version = PEP440VersioningScheme.parse_version("1.2.3")

            cmd.verify_version(version)


class ProjectFilePythonVersionCommandTestCase(unittest.TestCase):
    def test_project_file_not_found(self):
        with temp_directory(change_into=True):
            cmd = PythonVersionCommand(PEP440VersioningScheme)

            self.assertFalse(cmd.project_found())

    def test_project_file_found(self):
        with temp_file(name="pyproject.toml", change_into=True):
            cmd = PythonVersionCommand(PEP440VersioningScheme)

            self.assertTrue(cmd.project_found())
